/** 
 * Copyright (c) SpaceToad, 2011
 * http://www.mod-buildcraft.com
 * 
 * BuildCraft is distributed under the terms of the Minecraft Mod Public 
 * License 1.0, or MMPL. Please check the contents of the license located in
 * http://www.mod-buildcraft.com/MMPL-1.0.txt
 */

package net.minecraft.src.buildcraft.api;

import java.util.LinkedList;

import net.minecraft.src.Block;
import net.minecraft.src.BuildCraftCore;
import net.minecraft.src.TileEntity;

public class Trigger {
	
	public int id;
	
	public Trigger (int id) {
		this.id = id;
		BuildCraftAPI.triggers [id] = this;
	}
	
	public String getTexture () {
		return BuildCraftCore.triggerTextures;
	}
	
	public int getIndexInTexture () {
		return 0;
	}
	
	public boolean hasParameter () {
		return false;
	}
	
	public String getDescription() {
		return "";
	}
	
	public boolean isTriggerActive (TileEntity tile, TriggerParameter parameter) {
		return false;
	}
	
	public TriggerParameter createParameter () {
		return new TriggerParameter();
	}
	
	private static LinkedList <ITriggerProvider> providers = new LinkedList <ITriggerProvider> ();
	
	public static void registerTriggerProvider (ITriggerProvider provider) {
		if (provider != null && !providers.contains(provider)) {
			providers.add(provider);
		}
	}
	
	public static LinkedList <Trigger> getNeighborTriggers (Block block, TileEntity entity) {
		LinkedList <Trigger> triggers = new LinkedList <Trigger> ();
		
		for (ITriggerProvider provider : providers) {
			LinkedList <Trigger> toAdd = provider.getNeighborTriggers(block, entity);
		
			if (toAdd != null) {
				for (Trigger t : toAdd) {
					if (!triggers.contains(t)) {
						triggers.add(t);
					}
				}
			}
		}
		
		return triggers;
	}
	
	public static LinkedList <Trigger> getPipeTriggers (IPipe pipe) {
		LinkedList <Trigger> triggers = new LinkedList <Trigger> ();
		
		for (ITriggerProvider provider : providers) {
			LinkedList <Trigger> toAdd = provider.getPipeTriggers(pipe);
		
			if (toAdd != null) {
				for (Trigger t : toAdd) {
					if (!triggers.contains(t)) {
						triggers.add(t);
					}
				}
			}
		}
		
		return triggers;
	}
}
